package org.app.sudoku;

import java.io.File;

import org.app.algorithms.Recognizer;

import android.app.Activity;
import android.app.Dialog;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.Gravity;
import android.widget.Toast;

public class Game extends Activity {
	private static final String TAG = "Sudoku";

	public static final String KEY_DIFFICULTY =
			"org.example.sudoku.difficulty";
	private static final String PREF_PUZZLE = "puzzle" ;
	public static final int DIFFICULTY_EASY = 0;
	public static final int DIFFICULTY_MEDIUM = 1;
	public static final int DIFFICULTY_HARD = 2;
	protected static final int DIFFICULTY_CONTINUE = -1;

	private int puzzle[][];

	private PuzzleView puzzleView;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Log.d(TAG, "onCreate");

		puzzle = getPuzzle();
		calculateUsedTiles();

		puzzleView = new PuzzleView(this);
		setContentView(puzzleView);
		puzzleView.requestFocus();


		// If the activity is restarted, do a continue next time
		getIntent().putExtra(KEY_DIFFICULTY, DIFFICULTY_CONTINUE);
	}

	@Override
	protected void onResume() {
		super.onResume();
	}

	@Override
	protected void onPause() {
		super.onPause();
		Log.d(TAG, "onPause");
		// Save the current puzzle
		getPreferences(MODE_PRIVATE).edit().putString(PREF_PUZZLE,
				toPuzzleString(puzzle)).commit();
	}

	/** Given a difficulty level, come up with a new puzzle */
	private int[][] getPuzzle() {
		String imagePath = Environment
				.getExternalStorageDirectory().getAbsolutePath()
				+ File.separator + "myDir" + File.separator + "IMG_3421.jpg";

		return Recognizer.recognize(CameraCapture.sudokuPhotoPath);
	}

	/** Convert an array into a puzzle string */
	static private String toPuzzleString(int[][] puz) {
		StringBuilder buf = new StringBuilder();
		for (int i = 0; i < puz.length; i++) {
			for (int j = 0; j < puz[0].length; j++) {
				buf.append(puz[i][j]);
			}
		}
		return buf.toString();
	}

	/** Convert a puzzle string into an array */
	static protected int[][] fromPuzzleString(String string) {
		int[][] puz = new int[9][9];
		for (int i = 0; i < 9; i++) {
			for (int j = 0; j < 9; j++) {
				puz[i][j] = string.charAt(i) - '0';
			}
		}
		return puz;
	}

	/** Return a string for the tile at the given coordinates */
	protected String getTileString(int x, int y) {
		int v = puzzle[x][y];
		if (v == 0)
			return "";
		else
			return String.valueOf(v);
	}

	/** Change the tile only if it's a valid move */
	protected boolean setTileIfValid(int x, int y, int value) {
		int tiles[] = getUsedTiles(x, y);
		if (value != 0) {
			for (int tile : tiles) {
				if (tile == value)
					return false;
			}
		}
		puzzle[x][y] = value;
		calculateUsedTiles();
		return true;
	}

	protected void solve() {
		Recognizer.solve(puzzle);
		for (int i = 0; i < 9; i++) {
			for (int j = 0; j < 9; j++) {
				setTileIfValid(i, j, puzzle[i][j]);
			}
		}
		puzzleView.refresh();
	}

	/** Open the keypad if there are any valid moves */
	protected void showKeypadOrError(int x, int y) {
		int tiles[] = getUsedTiles(x, y);
		if (tiles.length == 9) {
			Toast toast = Toast.makeText(this,
					R.string.no_moves_label, Toast.LENGTH_SHORT);
			toast.setGravity(Gravity.CENTER, 0, 0);
			toast.show();
		} else {
			//Log.d(TAG, "showKeypad: used=" + toPuzzleString(tiles));
			Dialog v = new Keypad(this, tiles, puzzleView);
			v.show();
		}
	}

	/** Cache of used tiles */
	private final int used[][][] = new int[9][9][];

	/** Return cached used tiles visible from the given coords */
	protected int[] getUsedTiles(int x, int y) {
		return used[x][y];
	}

	/** Compute the two dimensional array of used tiles */
	private void calculateUsedTiles() {
		for (int x = 0; x < 9; x++) {
			for (int y = 0; y < 9; y++) {
				used[x][y] = calculateUsedTiles(x, y);
				// Log.d(TAG, "used[" + x + "][" + y + "] = "
				// + toPuzzleString(used[x][y]));
			}
		}
	}

	/** Compute the used tiles visible from this position */
	private int[] calculateUsedTiles(int x, int y) {
		int c[] = new int[9];
		// horizontal
		for (int i = 0; i < 9; i++) {
			if (i == x)
				continue;
			int t = puzzle[i][y];
			if (t != 0)
				c[t - 1] = t;
		}
		// vertical
		for (int i = 0; i < 9; i++) {
			if (i == y)
				continue;
			int t = puzzle[x][i];
			if (t != 0)
				c[t - 1] = t;
		}
		// same cell block
		int startx = (x / 3) * 3;
		int starty = (y / 3) * 3;
		for (int i = startx; i < startx + 3; i++) {
			for (int j = starty; j < starty + 3; j++) {
				if (i == x && j == y)
					continue;
				int t = puzzle[i][j];
				if (t != 0)
					c[t - 1] = t;
			}
		}
		// compress
		int nused = 0;
		for (int t : c) {
			if (t != 0)
				nused++;
		}
		int c1[] = new int[nused];
		nused = 0;
		for (int t : c) {
			if (t != 0)
				c1[nused++] = t;
		}
		return c1;
	}

}
